//
//  GameScene.m
//  SpriteKitTest
//
//  Created by David Corrigan on 5/1/16.
//  Copyright (c) 2016 Digital Hero Games, LLC. All rights reserved.
//

#import "GameScene.h"

@implementation GameScene

-(void)didMoveToView:(SKView *)view {
    /* Setup your scene here */
    
    //Change the background color of the scene
    self.backgroundColor = SKColor.blackColor;
    
    //Load the player's spaceship sprite
    spaceship = [SKSpriteNode spriteNodeWithImageNamed:@"spaceship_enemy"];
    spaceship.scale = 0.2;
    spaceship.position = CGPointMake(CGRectGetMidX(self.frame), (100));
    [self addChild:spaceship];
    
    
    //Create a label for the score
    myLabel = [SKLabelNode labelNodeWithFontNamed:@"Chalkduster"];
    
    myLabel.text = @"Score:";
    myLabel.fontSize = 20;
    myLabel.position = CGPointMake(CGRectGetMidX(self.frame),
                                   CGRectGetMaxY((self.frame))-50);
    
    [self addChild:myLabel];
    
    asteroidCount = 0;
    
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    /* Called when a touch begins */
    
    isShipMovingLeft = NO;
    isShipMovingLeft = NO;
    
    for (UITouch *touch in touches) {
        CGPoint location = [touch locationInNode:self];
        
        if (location.x > CGRectGetMidX(self.frame))
        {
            //We are on the right side of the screen
            //Move the spaceship right
            
            isShipMovingRight = YES;
            
        } else if (location.x < CGRectGetMidX(self.frame)){
            //We are on the left side of the screen
            //Move the spaceship left
            
            isShipMovingLeft = YES;
            
        }
    }
}

-(void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    isShipMovingRight = NO;
    isShipMovingLeft = NO;
    
}


-(void)createAsteroid
{
    //Create an asteroid and move it towards the player's ship
    
    //get random x value between screen bounds
    int xPosition = 0;
    
    xPosition = [self getRandomNumberBetween:1 to:(int)self.size.width];
    
    //Load the player's spaceship sprite
    SKSpriteNode* asteroid = [SKSpriteNode spriteNodeWithImageNamed:@"aestroid_dark"];
    asteroid.scale = 0.15;
    asteroid.name = @"aestroid_dark";
    asteroid.position = CGPointMake(xPosition, self.size.height + asteroid.size.height);
    [self addChild:asteroid];
    
    //move the asteroid to the bottom of the screen (y 0) along the same xPosition
    SKAction* moveAsteroid = [SKAction moveTo:(CGPointMake(xPosition, 0 - asteroid.size.height)) duration:(4.0)];
    [asteroid runAction:moveAsteroid];
    
    //add some rotation to the asteroids at random speeds
    SKAction *rotateAction = [SKAction rotateByAngle:M_PI duration:[self getRandomNumberBetween:1 to:5]];
    
    [asteroid runAction:[SKAction repeatActionForever:rotateAction]];
    
    asteroidCount++;
    
    waitingOnAsteroid = NO;
    
    
}

-(int)getRandomNumberBetween:(int)from to:(int)to {
    
    return (int)from + arc4random() % (to-from+1);
}


-(void)shipReset{
    isShipDamaged = NO;
}

-(void)update:(CFTimeInterval)currentTime {
    /* Called before each frame is rendered */
    
    
    //check to see if we need to create a new asteroid
    if (asteroidCount <= 10 && !waitingOnAsteroid)
    {
        waitingOnAsteroid = YES;
        
        int xTime = [self getRandomNumberBetween:1 to:5];

        [self performSelector:@selector(createAsteroid) withObject:nil afterDelay: (1 + xTime/10)];
        
    }
    
    //check to see if we need to move the spaceship
    if(isShipMovingRight){
    
        if(spaceship.position.x < self.size.width - spaceship.size.width/2){
        
            spaceship.position = CGPointMake(spaceship.position.x + 5, spaceship.position.y);
        }
        
    } else if (isShipMovingLeft){
        
        if(spaceship.position.x > spaceship.size.width/2){
            
            spaceship.position = CGPointMake(spaceship.position.x - 5, spaceship.position.y);
        }
    }
    
    
    //loop through all asteroids to check for collision or moving off screen
    [self enumerateChildNodesWithName:@"aestroid_dark" usingBlock:^(SKNode *node, BOOL *stop) {
        
        if([node intersectsNode:spaceship] && !isShipDamaged)
        {
            //ship collided with asteroid
            
            score = 0;
            
            myLabel.text = [NSString stringWithFormat:@"Score: %d", score];

            isShipDamaged = YES;
            
            //add an action to pulse the spaceship red
            SKAction *pulseRed = [SKAction sequence:@[
                [SKAction colorizeWithColor:[SKColor redColor] colorBlendFactor:1.0 duration:0.15],
                [SKAction waitForDuration:0.1],
                [SKAction colorizeWithColorBlendFactor:0.0 duration:0.15]]];
            
            [spaceship runAction: pulseRed];

            
            [self performSelector:@selector(shipReset) withObject:nil afterDelay: (1)];
            
        }
        
        //if asteroid went off the screen and past the spaceship
        if(node.position.y <= 0)
        {
            score += 100;
            
            myLabel.text = [NSString stringWithFormat:@"Score: %d", score];
            
            asteroidCount--;
            
            [node removeFromParent];
            
            
        }
    }];
    
    
}


    


@end
